#include "alarmsettingsdialog.h"
#include "ui_alarmsettingsdialog.h"

#include <QFileDialog>

AlarmSettingsDialog::AlarmSettingsDialog(AlarmTime* almNight, AlarmTime* almAlarm, QSettings* settings, QWidget *parent): QDialog(parent), ui(new Ui::AlarmSettingsDialog), almNight_(almNight), almAlarm_(almAlarm), settings_(settings)
{
    ui->setupUi(this);

    //restore settings
    ui->checkBox_Alarm->setChecked(settings_->value("Alarms/alarmOn").toBool());
    ui->checkBox_Sunrise->setChecked(settings_->value("Alarms/sunrise").toBool());
    ui->timeEdit_Shutdown->setTime(settings_->value("Alarms/shutdownTime").toTime());
    ui->timeEdit_Alarm->setTime(settings_->value("Alarms/alarmTime").toTime());
    ui->lineEdit->setText(settings_->value("Alarms/alarmSoundFile").toString());

    connect(ui->pushButton_changeFile, SIGNAL(clicked()), this, SLOT(showFileChooser()));
}

AlarmSettingsDialog::~AlarmSettingsDialog()
{
    delete ui;
}

void AlarmSettingsDialog::accept()
{
    //store settings
    settings_->setValue("Alarms/alarmOn", ui->checkBox_Alarm->checkState());
    settings_->setValue("Alarms/sunrise", ui->checkBox_Sunrise->checkState());
    settings_->setValue("Alarms/shutdownTime", ui->timeEdit_Shutdown->time());
    settings_->setValue("Alarms/alarmTime", ui->timeEdit_Alarm->time());
    settings_->setValue("Alarms/alarmSoundFile", ui->lineEdit->text());

    //send signals
    signalAlarmSoundFile(ui->lineEdit->text());
    signalSunrise(ui->checkBox_Sunrise->checkState());

    //modify alarm objects
    almAlarm_->changeTime(ui->timeEdit_Alarm->time());
    almNight_->changeTime(ui->timeEdit_Shutdown->time());

    QDialog::accept();
}

void AlarmSettingsDialog::showFileChooser()
{
    ui->lineEdit->setText(QFileDialog::getOpenFileName(this));
}


