#ifndef MAINWINDOW_H
#define MAINWINDOW_H

#include <QMainWindow>
#include <QColorDialog>
#include <QListWidgetItem>
#include <QTime>
#include <vector>
#include "alarmtime.h"
#include "microcontroller.h"


namespace Ui
{
class MainWindow;
}

class MainWindow : public QMainWindow
{
    Q_OBJECT

public:
    explicit MainWindow(Microcontroller *micro, bool isRemoteMode = false, QWidget *parent = nullptr);
    ~MainWindow();



private:
    Ui::MainWindow *ui;

    QColorDialog colorChooser;

    Microcontroller *_micro;

    std::vector<QAbstractButton*> _relayCheckBoxes;

    void remoteMode();

signals:

    void signalAmpOn();
    void signalAmpOff();

    void showAlmSettingsDialog();

    void showAdvRelayDialog();

private slots:

    //RGB
    void slotChangedRgb(const QColor color);
    void slotApplyPreset();

    void changeHeaderLableText(const QString string);

    //relays
    void relayCheckBoxToggeled(int state);

    //Automation
    void slotAmpAutoToggle(int state);
    void slotBSpeakerAutoToggle(int state);
    void slotInfMirrorAutoToggle(int state);

    //door
    void slotDoorOpenTimeout();

public slots:

    void relayStateChanged(std::vector<bool> relayStates);
    void auxStateChanged(int value);

};

#endif // MAINWINDOW_H
